using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;
using ELTE.TravelAgency.Service.Models;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Microsoft.OpenApi.Models;

namespace ELTE.TravelAgency.Service
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            // Dependency injection belltsa az adatbzis kontextushoz
            services.AddDbContext<TravelAgencyContext>(options =>
                options.UseSqlServer(Configuration.GetConnectionString("DefaultConnection")));

            // Dependency injection belltsa az authentikcihoz
            services.AddIdentity<Guest, IdentityRole<int>>()
                .AddEntityFrameworkStores<TravelAgencyContext>() // EF hasznlata a TravelAgencyContext entits kontextussal
                .AddDefaultTokenProviders(); // Alaprtelmezett token genertor hasznlata (pl. SecurityStamp-hez)

            services.AddControllers();

            // Swagger generator regisztrlsa
            services.AddSwaggerGen(c =>
            {
                // (nv, OpenApiInfo) prok megadsa szksges
                c.SwaggerDoc("v1", new OpenApiInfo {Title = "Travel Agency API", Version = "v1"});

                // XML API dokumentci felhasznlsa a Swaggerben
                var xmlFile = $"{Assembly.GetExecutingAssembly().GetName().Name}.xml";
                var xmlPath = Path.Combine(AppContext.BaseDirectory, xmlFile);
                c.IncludeXmlComments(xmlPath);
            });
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseRouting();

            app.UseAuthentication();

            app.UseAuthorization();

            // Swagger hasznlata (JSON vgpontok)
            app.UseSwagger();

            // Swagger UI engedlyezse (bngszhet HTML vgpontok)
            app.UseSwaggerUI(c =>
            {
                // a JSON vgpont megadsa (s engedlyezse szksges)
                c.SwaggerEndpoint("/swagger/v1/swagger.json", "Travel Agency API V1");
            });

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }
    }
}
